@@ -1,15 +1,21 @@
 /*
- * Copyright (C) 2004-2012  See the AUTHORS file for details.
+ * Copyright (C) 2004-2013 ZNC, see the NOTICE file for details.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 2 as published
- * by the Free Software Foundation.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 #include <znc/User.h>
 #include <znc/IRCNetwork.h>
-#include <znc/IRCSock.h>
-#include <znc/znc.h>
 
 using std::vector;
 
@@ -171,7 +177,7 @@ private:
 template<> void TModInfo<CBlockUser>(CModInfo& Info) {
 	Info.SetWikiPage("blockuser");
 	Info.SetHasArgs(true);
-	Info.SetArgsHelpText("Enter one or more user names. Seperate them by spaces.");
+	Info.SetArgsHelpText("Enter one or more user names. Separate them by spaces.");
 }
 
 GLOBALMODULEDEFS(CBlockUser, "Block certain users from logging in")
