@@ -1,9 +1,17 @@
 /*
- * Copyright (C) 2004-2012  See the AUTHORS file for details.
+ * Copyright (C) 2004-2013 ZNC, see the NOTICE file for details.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 2 as published
- * by the Free Software Foundation.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 #include <znc/Modules.h>
@@ -26,6 +34,15 @@ public:
 		DelNV("Password");
 	}
 
+	void SetNSNameCommand(const CString& sLine) {
+		SetNV("NickServName", sLine.Token(1, true));
+		PutModule("NickServ name set");
+	}
+
+	void ClearNSNameCommand(const CString& sLine) {
+		DelNV("NickServName");
+	}
+
 	void GhostCommand(const CString& sLine) {
 		if (sLine.Token(1).empty()) {
 			PutModule("Syntax: ghost <nickname>");
@@ -92,6 +109,10 @@ public:
 			"password");
 		AddCommand("Clear", static_cast<CModCommand::ModCmdFunc>(&CNickServ::ClearCommand),
 			"", "Clear your nickserv password");
+		AddCommand("SetNSName", static_cast<CModCommand::ModCmdFunc>(&CNickServ::SetNSNameCommand),
+			"nickname", "Set NickServ name (Useful on networks like EpiKnet, where NickServ is named Themis)");
+		AddCommand("ClearNSName", static_cast<CModCommand::ModCmdFunc>(&CNickServ::ClearNSNameCommand),
+			"", "Reset NickServ name to default (NickServ)");
 		AddCommand("Ghost", static_cast<CModCommand::ModCmdFunc>(&CNickServ::GhostCommand),
 			"nickname", "GHOST disconnects an old user session, or somebody attempting to use your nickname without authorization.");
 		AddCommand("Recover", static_cast<CModCommand::ModCmdFunc>(&CNickServ::RecoverCommand),
@@ -109,9 +130,9 @@ public:
 	virtual ~CNickServ() {}
 
 	virtual bool OnLoad(const CString& sArgs, CString& sMessage) {
-		if (!sArgs.empty()) {
+		if (!sArgs.empty() && sArgs != "<hidden>") {
 			SetNV("Password", sArgs);
-			SetArgs("");
+			SetArgs("<hidden>");
 		}
 
 		if (GetNV("IdentifyCmd").empty()) {
@@ -134,13 +155,15 @@ public:
 	}
 
 	void HandleMessage(CNick& Nick, const CString& sMessage) {
+		CString sNickServName = (!GetNV("NickServName").empty()) ? GetNV("NickServName") : "NickServ";
 		if (!GetNV("Password").empty()
-				&& Nick.GetNick().Equals("NickServ")
+				&& Nick.NickEquals(sNickServName)
 				&& (sMessage.find("msg") != CString::npos
 				 || sMessage.find("authenticate") != CString::npos
 				 || sMessage.find("choose a different nickname") != CString::npos
+				 || sMessage.find("If this is your nick, identify yourself with") != CString::npos
 				 || sMessage.find("If this is your nick, type") != CString::npos
-				 || sMessage.find("type /NickServ IDENTIFY password") != CString::npos)
+				 || sMessage.StripControls_n().find("type /NickServ IDENTIFY password") != CString::npos)
 				&& sMessage.AsUpper().find("IDENTIFY") != CString::npos
 				&& sMessage.find("help") == CString::npos) {
 			MCString msValues;
