@@ -1,14 +1,20 @@
 /*
- * Copyright (C) 2004-2012  See the AUTHORS file for details.
+ * Copyright (C) 2004-2013 ZNC, see the NOTICE file for details.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 2 as published
- * by the Free Software Foundation.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
-#include <znc/User.h>
 #include <znc/IRCNetwork.h>
-#include <algorithm>
 
 class CPerform : public CModule {
 	void Add(const CString& sCommand) {
@@ -40,22 +46,17 @@ class CPerform : public CModule {
 	void List(const CString& sCommand) {
 		CTable Table;
 		unsigned int index = 1;
-		CString sExpanded;
 
 		Table.AddColumn("Id");
 		Table.AddColumn("Perform");
 		Table.AddColumn("Expanded");
 
-		for (VCString::const_iterator it = m_vPerform.begin(); it != m_vPerform.end(); it++, index++) {
+		for (VCString::const_iterator it = m_vPerform.begin(); it != m_vPerform.end(); ++it, index++) {
 			Table.AddRow();
 			Table.SetCell("Id", CString(index));
 			Table.SetCell("Perform", *it);
 
-			if (m_pNetwork) {
-				sExpanded = m_pNetwork->ExpandString(*it);
-			} else {
-				sExpanded = GetUser()->ExpandString(*it);
-			}
+			CString sExpanded = ExpandString(*it);
 
 			if (sExpanded != *it) {
 				Table.SetCell("Expanded", sExpanded);
@@ -128,7 +129,7 @@ public:
 
 	virtual void OnIRCConnected() {
 		for (VCString::const_iterator it = m_vPerform.begin(); it != m_vPerform.end(); ++it) {
-			PutIRC(m_pNetwork->ExpandString(*it));
+			PutIRC(ExpandString(*it));
 		}
 	}
 
