@@ -1,16 +1,22 @@
 /*
- * Copyright (C) 2008-2012  See the AUTHORS file for details.
+ * Copyright (C) 2004-2013 ZNC, see the NOTICE file for details.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 2 as published
- * by the Free Software Foundation.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
-#include <znc/Modules.h>
 #include <znc/User.h>
 #include <znc/IRCNetwork.h>
 #include <znc/IRCSock.h>
-#include <znc/Nick.h>
 #include <znc/Chan.h>
 
 using std::set;
@@ -306,7 +312,7 @@ private:
 	}
 
 	EModRet HandleMessage(const CNick& Nick, CString sMessage) {
-		if (!Nick.GetNick().Equals("Q") || !Nick.GetHost().Equals("CServe.quakenet.org"))
+		if (!Nick.NickEquals("Q") || !Nick.GetHost().Equals("CServe.quakenet.org"))
 			return CONTINUE;
 
 		sMessage.Trim();
@@ -412,7 +418,7 @@ private:
 	}
 
 	bool IsSelf(const CNick& Nick) {
-		return Nick.GetNick().Equals(m_pNetwork->GetCurNick());
+		return Nick.NickEquals(m_pNetwork->GetCurNick());
 	}
 
 	bool PackHex(const CString& sHex, CString& sPackedHex) {
