@@ -1,12 +1,19 @@
 /*
- * Copyright (C) 2004-2012  See the AUTHORS file for details.
+ * Copyright (C) 2004-2013 ZNC, see the NOTICE file for details.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 2 as published
- * by the Free Software Foundation.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
-#include <znc/znc.h>
 #include <znc/IRCNetwork.h>
 #include <znc/IRCSock.h>
 
@@ -60,9 +67,13 @@ static const struct {
 		{"313", false},  /* rfc1459 RPL_WHOISOPERATOR */
 		{"317", false},  /* rfc1459 RPL_WHOISIDLE */
 		{"319", false},  /* rfc1459 RPL_WHOISCHANNELS */
-		// "<ip> :actually using host"
-		{"338", false},
 		{"301", false},  /* rfc1459 RPL_AWAY */
+		{"276", false},  /* oftc-hybrid RPL_WHOISCERTFP */
+		{"330", false},  /* ratbox RPL_WHOISLOGGEDIN
+		                    aka ircu RPL_WHOISACCOUNT */
+		{"338", false},  /* RPL_WHOISACTUALLY -- "actually using host" */
+		{"378", false},  /* RPL_WHOISHOST -- real address of vhosts */
+		{"671", false},  /* RPL_WHOISSECURE */
 		{"318", true},  /* rfc1459 RPL_ENDOFWHOIS */
 		{"401", true},  /* rfc1459 ERR_NOSUCHNICK */
 		{"402", true},  /* rfc1459 ERR_NOSUCHSERVER */
