@@ -1,18 +1,25 @@
 /*
- * Copyright (C) 2004-2012  See the AUTHORS file for details.
+ * Copyright (C) 2004-2013 ZNC, see the NOTICE file for details.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 2 as published
- * by the Free Software Foundation.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 #include <znc/Chan.h>
-#include <znc/User.h>
 #include <znc/IRCNetwork.h>
-#include <list>
 
 using std::list;
 using std::vector;
+using std::set;
 
 class CWatchSource {
 public:
@@ -66,7 +73,7 @@ public:
 
 		bool bGoodSource = true;
 
-		if (sSource.size() && m_vsSources.size()) {
+		if (!sSource.empty() && !m_vsSources.empty()) {
 			bGoodSource = false;
 
 			for (unsigned int a = 0; a < m_vsSources.size(); a++) {
@@ -287,15 +294,19 @@ public:
 
 private:
 	void Process(const CNick& Nick, const CString& sMessage, const CString& sSource) {
+		set<CString> sHandledTargets;
+
 		for (list<CWatchEntry>::iterator it = m_lsWatchers.begin(); it != m_lsWatchers.end(); ++it) {
 			CWatchEntry& WatchEntry = *it;
 
-			if (WatchEntry.IsMatch(Nick, sMessage, sSource, m_pNetwork)) {
+			if (WatchEntry.IsMatch(Nick, sMessage, sSource, m_pNetwork) &&
+				sHandledTargets.count(WatchEntry.GetTarget()) < 1) {
 				if (m_pNetwork->IsUserAttached()) {
 					m_pNetwork->PutUser(":" + WatchEntry.GetTarget() + "!watch@znc.in PRIVMSG " + m_pNetwork->GetCurNick() + " :" + sMessage);
 				} else {
 					m_Buffer.AddLine(":" + _NAMEDFMT(WatchEntry.GetTarget()) + "!watch@znc.in PRIVMSG {target} :{text}", sMessage);
 				}
+				sHandledTargets.insert(WatchEntry.GetTarget());
 			}
 		}
 	}
@@ -319,7 +330,7 @@ private:
 
 		list<CWatchEntry>::iterator it = m_lsWatchers.begin();
 		for (unsigned int a = 0; a < uIdx; a++)
-			it++;
+			++it;
 
 		(*it).SetDisabled(bDisabled);
 		PutModule("Id " + CString(uIdx +1) + ((bDisabled) ? " Disabled" : " Enabled"));
@@ -337,7 +348,7 @@ private:
 
 		unsigned int uIdx = 1;
 
-		for (list<CWatchEntry>::iterator it = m_lsWatchers.begin(); it != m_lsWatchers.end(); it++, uIdx++) {
+		for (list<CWatchEntry>::iterator it = m_lsWatchers.begin(); it != m_lsWatchers.end(); ++it, uIdx++) {
 			CWatchEntry& WatchEntry = *it;
 
 			Table.AddRow();
@@ -367,7 +378,7 @@ private:
 
 		unsigned int uIdx = 1;
 
-		for (list<CWatchEntry>::iterator it = m_lsWatchers.begin(); it != m_lsWatchers.end(); it++, uIdx++) {
+		for (list<CWatchEntry>::iterator it = m_lsWatchers.begin(); it != m_lsWatchers.end(); ++it, uIdx++) {
 			CWatchEntry& WatchEntry = *it;
 
 			PutModule("/msg " + GetModNick() + " ADD " + WatchEntry.GetHostMask() + " " +
@@ -395,7 +406,7 @@ private:
 
 		list<CWatchEntry>::iterator it = m_lsWatchers.begin();
 		for (unsigned int a = 0; a < uIdx; a++)
-			it++;
+			++it;
 
 		(*it).SetSources(sSources);
 		PutModule("Sources set for Id " + CString(uIdx +1) + ".");
@@ -411,7 +422,7 @@ private:
 
 		list<CWatchEntry>::iterator it = m_lsWatchers.begin();
 		for (unsigned int a = 0; a < uIdx; a++)
-			it++;
+			++it;
 
 		m_lsWatchers.erase(it);
 		PutModule("Id " + CString(uIdx +1) + " Removed.");
