@@ -1,24 +1,24 @@
 /*
- * Copyright (C) 2004-2012  See the AUTHORS file for details.
+ * Copyright (C) 2004-2013 ZNC, see the NOTICE file for details.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 2 as published
- * by the Free Software Foundation.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 #include <znc/Chan.h>
-#include <znc/HTTPSock.h>
 #include <znc/Server.h>
-#include <znc/Template.h>
 #include <znc/User.h>
-#include <znc/znc.h>
-#include <znc/WebModules.h>
-#include <znc/ZNCString.h>
-#include <znc/Listener.h>
 #include <znc/IRCNetwork.h>
 #include <znc/IRCSock.h>
-#include <sstream>
-#include <utility>
 
 using std::stringstream;
 using std::make_pair;
@@ -35,25 +35,40 @@ using std::map;
    }
 */
 struct FOR_EACH_MODULE_Type {
-	bool bOnCMuser;
+	enum {
+		AtGlobal,
+		AtUser,
+		AtNetwork,
+	} where;
 	CModules CMtemp;
 	CModules& CMuser;
-	FOR_EACH_MODULE_Type(CUser* pUser) : CMuser(pUser ? pUser->GetModules() : CMtemp) {
-		bOnCMuser = false;
+	CModules& CMnet;
+	FOR_EACH_MODULE_Type(CUser* pUser) : CMuser(pUser ? pUser->GetModules() : CMtemp), CMnet(CMtemp) {
+		where = AtGlobal;
+	}
+	FOR_EACH_MODULE_Type(CIRCNetwork* pNetwork) : CMuser(pNetwork ? pNetwork->GetUser()->GetModules() : CMtemp), CMnet(pNetwork ? pNetwork->GetModules() : CMtemp) {
+		where = AtGlobal;
+	}
+	FOR_EACH_MODULE_Type(std::pair<CUser*, CIRCNetwork*> arg) : CMuser(arg.first ? arg.first->GetModules() : CMtemp), CMnet(arg.second ? arg.second->GetModules() : CMtemp) {
+		where = AtGlobal;
 	}
 	operator bool() { return false; }
 };
 
 inline bool FOR_EACH_MODULE_CanContinue(FOR_EACH_MODULE_Type& state, CModules::iterator& i) {
-	if (!state.bOnCMuser && i == CZNC::Get().GetModules().end()) {
+	if (state.where == FOR_EACH_MODULE_Type::AtGlobal && i == CZNC::Get().GetModules().end()) {
 		i = state.CMuser.begin();
-		state.bOnCMuser = true;
+		state.where = FOR_EACH_MODULE_Type::AtUser;
 	}
-	return !(state.bOnCMuser && i == state.CMuser.end());
+	if (state.where == FOR_EACH_MODULE_Type::AtUser && i == state.CMuser.end()) {
+		i = state.CMnet.begin();
+		state.where = FOR_EACH_MODULE_Type::AtNetwork;
+	}
+	return !(state.where == FOR_EACH_MODULE_Type::AtNetwork && i == state.CMnet.end());
 }
 
-#define FOR_EACH_MODULE(I, pUser)\
-	if (FOR_EACH_MODULE_Type FOR_EACH_MODULE_Var = pUser) {} else\
+#define FOR_EACH_MODULE(I, pUserOrNetwork)\
+	if (FOR_EACH_MODULE_Type FOR_EACH_MODULE_Var = pUserOrNetwork) {} else\
 	for (CModules::iterator I = CZNC::Get().GetModules().begin(); FOR_EACH_MODULE_CanContinue(FOR_EACH_MODULE_Var, I); ++I)
 
 class CWebAdminMod : public CModule {
@@ -72,7 +87,7 @@ public:
 	}
 
 	virtual bool OnLoad(const CString& sArgStr, CString& sMessage) {
-		if (sArgStr.empty())
+		if (sArgStr.empty() || CModInfo::GlobalModule != GetType())
 			return true;
 
 		// We don't accept any arguments, but for backwards
@@ -203,7 +218,6 @@ public:
 		sArg = WebSock.GetParam("altnick"); if (!sArg.empty()) { pNewUser->SetAltNick(sArg); }
 		sArg = WebSock.GetParam("statusprefix"); if (!sArg.empty()) { pNewUser->SetStatusPrefix(sArg); }
 		sArg = WebSock.GetParam("ident"); if (!sArg.empty()) { pNewUser->SetIdent(sArg); }
-		sArg = WebSock.GetParam("skin"); if (!sArg.empty()) { pNewUser->SetSkinName(sArg); }
 		sArg = WebSock.GetParam("realname"); if (!sArg.empty()) { pNewUser->SetRealName(sArg); }
 		sArg = WebSock.GetParam("quitmsg"); if (!sArg.empty()) { pNewUser->SetQuitMsg(sArg); }
 		sArg = WebSock.GetParam("chanmodes"); if (!sArg.empty()) { pNewUser->SetDefaultChanModes(sArg); }
@@ -262,6 +276,7 @@ public:
 		pNewUser->SetTimestampPrepend(WebSock.GetParam("prependtimestamp").ToBool());
 		pNewUser->SetTimezone(WebSock.GetParam("timezone"));
 		pNewUser->SetJoinTries(WebSock.GetParam("jointries").ToUInt());
+		pNewUser->SetMaxJoins(WebSock.GetParam("maxjoins").ToUInt());
 
 		if (spSession->IsAdmin()) {
 			pNewUser->SetDenyLoadMod(WebSock.GetParam("denyloadmod").ToBool());
@@ -284,6 +299,21 @@ public:
 		if (spSession->IsAdmin() || (pUser && !pUser->DenyLoadMod())) {
 			WebSock.GetParamValues("loadmod", vsArgs);
 
+			// disallow unload webadmin from itself
+			if (CModInfo::UserModule == GetType() && pUser == CZNC::Get().FindUser(WebSock.GetUser())) {
+				bool bLoadedWebadmin = false;
+				for (a = 0; a < vsArgs.size(); ++a) {
+					CString sModName = vsArgs[a].TrimRight_n("\r");
+					if (sModName == GetModName()) {
+						bLoadedWebadmin = true;
+						break;
+					}
+				}
+				if (!bLoadedWebadmin) {
+					vsArgs.push_back(GetModName());
+				}
+			}
+
 			for (a = 0; a < vsArgs.size(); a++) {
 				CString sModRet;
 				CString sModName = vsArgs[a].TrimRight_n("\r");
@@ -404,7 +434,7 @@ public:
 			CIRCNetwork* pNetwork = SafeGetNetworkFromParam(WebSock);
 
 			// Admin||Self Check
-			if (!spSession->IsAdmin() && (!spSession->GetUser() || spSession->GetUser() != pNetwork->GetUser())) {
+			if (!spSession->IsAdmin() && (!spSession->GetUser() || !pNetwork || spSession->GetUser() != pNetwork->GetUser())) {
 				return false;
 			}
 
@@ -433,7 +463,7 @@ public:
 			CIRCNetwork* pNetwork = SafeGetNetworkFromParam(WebSock);
 
 			// Admin||Self Check
-			if (!spSession->IsAdmin() && (!spSession->GetUser() || spSession->GetUser() != pNetwork->GetUser())) {
+			if (!spSession->IsAdmin() && (!spSession->GetUser() || !pNetwork || spSession->GetUser() != pNetwork->GetUser())) {
 				return false;
 			}
 
@@ -457,7 +487,7 @@ public:
 			CIRCNetwork* pNetwork = SafeGetNetworkFromParam(WebSock);
 
 			// Admin||Self Check
-			if (!spSession->IsAdmin() && (!spSession->GetUser() || spSession->GetUser() != pNetwork->GetUser())) {
+			if (!spSession->IsAdmin() && (!spSession->GetUser() || !pNetwork || spSession->GetUser() != pNetwork->GetUser())) {
 				return false;
 			}
 
@@ -471,7 +501,7 @@ public:
 			CIRCNetwork* pNetwork = SafeGetNetworkFromParam(WebSock);
 
 			// Admin||Self Check
-			if (!spSession->IsAdmin() && (!spSession->GetUser() || spSession->GetUser() != pNetwork->GetUser())) {
+			if (!spSession->IsAdmin() && (!spSession->GetUser() || !pNetwork || spSession->GetUser() != pNetwork->GetUser())) {
 				return false;
 			}
 
@@ -616,7 +646,7 @@ public:
 			o4["DisplayName"] = "Disabled";
 			if (pChan && pChan->IsDisabled()) { o4["Checked"] = "true"; }
 
-			FOR_EACH_MODULE(i, pUser) {
+			FOR_EACH_MODULE(i, pNetwork) {
 				CTemplate& mod = Tmpl.AddRow("EmbeddedModuleLoop");
 				mod.insert(Tmpl.begin(), Tmpl.end());
 				mod["WebadminAction"] = "display";
@@ -671,7 +701,7 @@ public:
 		TmplMod["User"] = pUser->GetUserName();
 		TmplMod["ChanName"] = sChanName;
 		TmplMod["WebadminAction"] = "change";
-		FOR_EACH_MODULE(it, pUser) {
+		FOR_EACH_MODULE(it, pNetwork) {
 			(*it)->OnEmbeddedWebRequest(WebSock, "webadmin/channel", TmplMod);
 		}
 
@@ -803,6 +833,16 @@ public:
 				Tmpl["FloodBurst"] = "4";
 			}
 
+			FOR_EACH_MODULE(i, make_pair(pUser, pNetwork)) {
+				CTemplate& mod = Tmpl.AddRow("EmbeddedModuleLoop");
+				mod.insert(Tmpl.begin(), Tmpl.end());
+				mod["WebadminAction"] = "display";
+				if ((*i)->OnEmbeddedWebRequest(WebSock, "webadmin/network", mod)) {
+					mod["Embed"] = WebSock.FindTmpl(*i, "WebadminNetwork.tmpl");
+					mod["ModName"] = (*i)->GetModName();
+				}
+			}
+
 			return true;
 		}
 
@@ -821,9 +861,10 @@ public:
 				WebSock.PrintErrorPage("Network name should be alphanumeric");
 				return true;
 			}
-			pNetwork = pUser->AddNetwork(sName);
+			CString sNetworkAddError;
+			pNetwork = pUser->AddNetwork(sName, sNetworkAddError);
 			if (!pNetwork) {
-				WebSock.PrintErrorPage("Network [" + sName.Token(0) + "] already exists");
+				WebSock.PrintErrorPage(sNetworkAddError);
 				return true;
 			}
 		}
@@ -926,10 +967,18 @@ public:
 			}
 		}
 
-		for (set<CString>::iterator it2 = ssUnloadMods.begin(); it2 != ssUnloadMods.end(); it2++) {
+		for (set<CString>::iterator it2 = ssUnloadMods.begin(); it2 != ssUnloadMods.end(); ++it2) {
 			pNetwork->GetModules().UnloadModule(*it2);
 		}
 
+		CTemplate TmplMod;
+		TmplMod["Username"] = pUser->GetUserName();
+		TmplMod["Name"] = pNetwork->GetName();
+		TmplMod["WebadminAction"] = "change";
+		FOR_EACH_MODULE(it, make_pair(pUser, pNetwork)) {
+			(*it)->OnEmbeddedWebRequest(WebSock, "webadmin/network", TmplMod);
+		}
+
 		if (!CZNC::Get().WriteConfig()) {
 			WebSock.PrintErrorPage("Network added/modified, but config was not written");
 			return true;
@@ -991,7 +1040,7 @@ public:
 			return true;
 		}
 
-		WebSock.Redirect("edituser?user=" + pNetwork->GetUser()->GetUserName().Escape_n(CString::EURL) + "&network=" + pNetwork->GetName().Escape_n(CString::EURL));
+		WebSock.Redirect("editnetwork?user=" + pNetwork->GetUser()->GetUserName().Escape_n(CString::EURL) + "&network=" + pNetwork->GetName().Escape_n(CString::EURL));
 		return false;
 	}
 
@@ -1033,6 +1082,7 @@ public:
 				Tmpl["Timezone"] = pUser->GetTimezone();
 				Tmpl["JoinTries"] = CString(pUser->JoinTries());
 				Tmpl["MaxNetworks"] = CString(pUser->MaxNetworks());
+				Tmpl["MaxJoins"] = CString(pUser->MaxJoins());
 
 				const set<CString>& ssAllowedHosts = pUser->GetAllowedHosts();
 				for (set<CString>::const_iterator it = ssAllowedHosts.begin(); it != ssAllowedHosts.end(); ++it) {
@@ -1054,7 +1104,7 @@ public:
 				}
 
 				const MCString& msCTCPReplies = pUser->GetCTCPReplies();
-				for (MCString::const_iterator it2 = msCTCPReplies.begin(); it2 != msCTCPReplies.end(); it2++) {
+				for (MCString::const_iterator it2 = msCTCPReplies.begin(); it2 != msCTCPReplies.end(); ++it2) {
 					CTemplate& l = Tmpl.AddRow("CTCPLoop");
 					l["CTCP"] = it2->first + " " + it2->second;
 				}
@@ -1149,6 +1199,9 @@ public:
 				if (pModule) {
 					l["Checked"] = "true";
 					l["Args"] = pModule->GetArgs();
+					if (CModInfo::UserModule == GetType() && Info.GetName() == GetModName()) {
+						l["Disabled"] = "true";
+					}
 				}
 
 				if (!spSession->IsAdmin() && pUser && pUser->DenyLoadMod()) {
@@ -1220,6 +1273,7 @@ public:
 
 		CUser* pNewUser = GetNewUser(WebSock, pUser);
 		if (!pNewUser) {
+			WebSock.PrintErrorPage("Invalid user settings");
 			return true;
 		}
 
@@ -1544,10 +1598,9 @@ public:
 				if (pModule) {
 					l["Checked"] = "true";
 					l["Args"] = pModule->GetArgs();
-				}
-
-				if (Info.GetName() == GetModName()) {
-					l["Disabled"] = "true";
+					if (CModInfo::GlobalModule == GetType() && Info.GetName() == GetModName()) {
+						l["Disabled"] = "true";
+					}
 				}
 
 				l["Name"] = Info.GetName();
@@ -1621,12 +1674,13 @@ public:
 		for (a = 0; a < vCurMods.size(); a++) {
 			CModule* pCurMod = vCurMods[a];
 
-			if (ssArgs.find(pCurMod->GetModName()) == ssArgs.end() && pCurMod->GetModName() != GetModName()) {
+			if (ssArgs.find(pCurMod->GetModName()) == ssArgs.end() &&
+					(CModInfo::GlobalModule != GetType() || pCurMod->GetModName() != GetModName())) {
 				ssUnloadMods.insert(pCurMod->GetModName());
 			}
 		}
 
-		for (set<CString>::iterator it2 = ssUnloadMods.begin(); it2 != ssUnloadMods.end(); it2++) {
+		for (set<CString>::iterator it2 = ssUnloadMods.begin(); it2 != ssUnloadMods.end(); ++it2) {
 			CZNC::Get().GetModules().UnloadModule(*it2);
 		}
 
@@ -1641,6 +1695,7 @@ public:
 };
 
 template<> void TModInfo<CWebAdminMod>(CModInfo& Info) {
+	Info.AddType(CModInfo::UserModule);
 	Info.SetWikiPage("webadmin");
 }
 
