@@ -1,17 +1,23 @@
 /*
- * Copyright (C) 2004-2012  See the AUTHORS file for details.
+ * Copyright (C) 2004-2013 ZNC, see the NOTICE file for details.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 2 as published
- * by the Free Software Foundation.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 #include <znc/Chan.h>
-#include <znc/FileUtils.h>
 #include <znc/IRCSock.h>
 #include <znc/User.h>
 #include <znc/IRCNetwork.h>
-#include <znc/znc.h>
 #include <znc/Config.h>
 
 using std::set;
@@ -280,13 +286,13 @@ void CChan::ModeChange(const CString& sModes, const CNick* pOpNick) {
 					if (bAdd) {
 						pNick->AddPerm(uPerm);
 
-						if (pNick->GetNick().Equals(m_pNetwork->GetCurNick())) {
+						if (pNick->NickEquals(m_pNetwork->GetCurNick())) {
 							AddPerm(uPerm);
 						}
 					} else {
 						pNick->RemPerm(uPerm);
 
-						if (pNick->GetNick().Equals(m_pNetwork->GetCurNick())) {
+						if (pNick->NickEquals(m_pNetwork->GetCurNick())) {
 							RemPerm(uPerm);
 						}
 					}
@@ -332,15 +338,17 @@ void CChan::ModeChange(const CString& sModes, const CNick* pOpNick) {
 					break;
 			}
 
-			bool bNoChange;
-			if (bList) {
-				bNoChange = false;
-			} else if (bAdd) {
-				bNoChange = HasMode(uMode) && GetModeArg(uMode) == sArg;
-			} else {
-				bNoChange = !HasMode(uMode);
+			if (pOpNick) {
+				bool bNoChange;
+				if (bList) {
+					bNoChange = false;
+				} else if (bAdd) {
+					bNoChange = HasMode(uMode) && GetModeArg(uMode) == sArg;
+				} else {
+					bNoChange = !HasMode(uMode);
+				}
+				NETWORKMODULECALL(OnMode(*pOpNick, *this, uMode, sArg, bAdd, bNoChange), m_pNetwork->GetUser(), m_pNetwork, NULL, NOTHING);
 			}
-			NETWORKMODULECALL(OnMode(*pOpNick, *this, uMode, sArg, bAdd, bNoChange), m_pNetwork->GetUser(), m_pNetwork, NULL, NOTHING);
 
 			if (!bList) {
 				(bAdd) ? AddMode(uMode, sArg) : RemMode(uMode);
@@ -456,7 +464,7 @@ bool CChan::AddNick(const CString& sNick) {
 		pNick->AddPerm(sPrefix[i]);
 	}
 
-	if (pNick->GetNick().Equals(m_pNetwork->GetCurNick())) {
+	if (pNick->NickEquals(m_pNetwork->GetCurNick())) {
 		for (CString::size_type i = 0; i < sPrefix.length(); i++) {
 			AddPerm(sPrefix[i]);
 		}
