@@ -1,19 +1,23 @@
 /*
- * Copyright (C) 2004-2012  See the AUTHORS file for details.
+ * Copyright (C) 2004-2013 ZNC, see the NOTICE file for details.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 2 as published
- * by the Free Software Foundation.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
-#include <znc/Client.h>
 #include <znc/Chan.h>
-#include <znc/FileUtils.h>
 #include <znc/IRCSock.h>
 #include <znc/User.h>
 #include <znc/IRCNetwork.h>
-#include <znc/znc.h>
-#include <znc/WebModules.h>
 
 using std::vector;
 
@@ -24,7 +28,7 @@ using std::vector;
 			pModule->SetClient(CLIENT);  \
 			pModule->FUNC;  \
 			pModule->SetClient(NULL);  \
-		} catch (CModule::EModException e) {  \
+		} catch (const CModule::EModException& e) {  \
 			if (e == CModule::UNLOAD) {  \
 				(NETWORK)->GetModules().UnloadModule(MOD);  \
 			}  \
@@ -36,7 +40,7 @@ using std::vector;
 			pModule->FUNC;  \
 			pModule->SetClient(NULL);  \
 			pModule->SetNetwork(NULL);  \
-		} catch (CModule::EModException e) {  \
+		} catch (const CModule::EModException& e) {  \
 			if (e == CModule::UNLOAD) {  \
 				(USER)->GetModules().UnloadModule(MOD);  \
 			}  \
@@ -50,7 +54,7 @@ using std::vector;
 			pModule->SetClient(NULL);  \
 			pModule->SetNetwork(NULL);  \
 			pModule->SetUser(NULL);  \
-		} catch (CModule::EModException e) {  \
+		} catch (const CModule::EModException& e) {  \
 			if (e == CModule::UNLOAD) {  \
 					CZNC::Get().GetModules().UnloadModule(MOD);  \
 			}  \
@@ -380,20 +384,31 @@ void CClient::ReadLine(const CString& sData) {
 	}
 
 	if (sCommand.Equals("DETACH")) {
-		CString sChan = sLine.Token(1);
+		CString sChannels = sLine.Token(1).TrimPrefix_n();
 
-		if (sChan.empty()) {
+		if (sChannels.empty()) {
 			PutStatusNotice("Usage: /detach <#chan>");
 			return;
 		}
 
-		CChan* pChan = m_pNetwork->FindChan(sChan);
-		if (!pChan) {
-			PutStatusNotice("You are not on [" + sChan + "]");
-			return;
+		VCString vChans;
+		sChannels.Split(",", vChans, false);
+		sChannels.clear();
+
+		for (VCString::const_iterator channelIterator = vChans.begin();
+				channelIterator != vChans.end();
+				++channelIterator)
+		{
+			CString sChannel = *channelIterator;
+
+			CChan *pChannel = m_pNetwork->FindChan(sChannel);
+			if (pChannel) {
+				pChannel->DetachUser();
+			} else {
+				PutStatusNotice("You are not on [" + sChannel + "]");
+			}
 		}
 
-		pChan->DetachUser();
 		return;
 	} else if (sCommand.Equals("JOIN")) {
 		CString sChans = sLine.Token(1).TrimPrefix_n();
@@ -767,12 +782,18 @@ void CClient::PutModNotice(const CString& sModule, const CString& sLine) {
 }
 
 void CClient::PutModule(const CString& sModule, const CString& sLine) {
+	VCString vsLines;
+	VCString::iterator it;
 	if (!m_pUser) {
 		return;
 	}
 
 	DEBUG("(" << GetFullName() << ") ZNC -> CLI [:" + m_pUser->GetStatusPrefix() + ((sModule.empty()) ? "status" : sModule) + "!znc@znc.in PRIVMSG " << GetNick() << " :" << sLine << "]");
-	Write(":" + m_pUser->GetStatusPrefix() + ((sModule.empty()) ? "status" : sModule) + "!znc@znc.in PRIVMSG " + GetNick() + " :" + sLine + "\r\n");
+
+	sLine.Split("\n", vsLines);
+	for (it = vsLines.begin(); it != vsLines.end(); ++it) {
+		Write(":" + m_pUser->GetStatusPrefix() + ((sModule.empty()) ? "status" : sModule) + "!znc@znc.in PRIVMSG " + GetNick() + " :" + (*it) + "\r\n");
+	}
 }
 
 CString CClient::GetNick(bool bAllowIRCNick) const {
@@ -816,7 +837,7 @@ void CClient::HandleCap(const CString& sLine)
 		for (SCString::iterator i = ssOfferCaps.begin(); i != ssOfferCaps.end(); ++i) {
 			sRes += *i + " ";
 		}
-		RespondCap("LS :" + sRes + "userhost-in-names multi-prefix znc.in/server-time");
+		RespondCap("LS :" + sRes + "userhost-in-names multi-prefix znc.in/server-time-iso");
 		m_bInCap = true;
 	} else if (sSubCmd.Equals("END")) {
 		m_bInCap = false;
@@ -838,7 +859,7 @@ void CClient::HandleCap(const CString& sLine)
 			if (sCap.TrimPrefix("-"))
 				bVal = false;
 
-			bool bAccepted = ("multi-prefix" == sCap) || ("userhost-in-names" == sCap) || ("znc.in/server-time" == sCap);
+			bool bAccepted = ("multi-prefix" == sCap) || ("userhost-in-names" == sCap) || ("znc.in/server-time-iso" == sCap);
 			GLOBALMODULECALL(IsClientCapSupported(this, sCap, bVal), &bAccepted);
 
 			if (!bAccepted) {
@@ -858,7 +879,7 @@ void CClient::HandleCap(const CString& sLine)
 				m_bNamesx = bVal;
 			} else if ("userhost-in-names" == *it) {
 				m_bUHNames = bVal;
-			} else if ("znc.in/server-time" == *it) {
+			} else if ("znc.in/server-time-iso" == *it) {
 				m_bServerTime = bVal;
 			}
 			GLOBALMODULECALL(OnClientCapRequest(this, *it, bVal), NOTHING);
@@ -897,7 +918,7 @@ void CClient::HandleCap(const CString& sLine)
 		}
 		if (m_bServerTime) {
 			m_bServerTime = false;
-			ssRemoved.insert("znc.in/server-time");
+			ssRemoved.insert("znc.in/server-time-iso");
 		}
 		CString sList = "";
 		for (SCString::iterator i = ssRemoved.begin(); i != ssRemoved.end(); ++i) {
