@@ -1,20 +1,25 @@
 /*
- * Copyright (C) 2004-2012  See the AUTHORS file for details.
+ * Copyright (C) 2004-2013 ZNC, see the NOTICE file for details.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 2 as published
- * by the Free Software Foundation.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
-#include <znc/Client.h>
 #include <znc/Chan.h>
 #include <znc/FileUtils.h>
 #include <znc/IRCNetwork.h>
 #include <znc/IRCSock.h>
-#include <znc/Listener.h>
 #include <znc/Server.h>
 #include <znc/User.h>
-#include <znc/znc.h>
 
 using std::vector;
 using std::set;
@@ -66,7 +71,7 @@ void CClient::UserCommand(CString& sLine) {
 		CIRCSock* pIRCSock = m_pNetwork->GetIRCSock();
 		const CString& sPerms = (pIRCSock) ? pIRCSock->GetPerms() : "";
 
-		if (!msNicks.size()) {
+		if (msNicks.empty()) {
 			PutStatus("No nicks on [" + sChan + "]");
 			return;
 		}
@@ -106,7 +111,7 @@ void CClient::UserCommand(CString& sLine) {
 			return;
 		}
 
-		CString sChan = sLine.Token(1);
+		CString sChan = sLine.Token(1).MakeLower();
 
 		if (sChan.empty()) {
 			PutStatus("Usage: Detach <#chan>");
@@ -117,7 +122,10 @@ void CClient::UserCommand(CString& sLine) {
 		vector<CChan*>::const_iterator it;
 		unsigned int uMatches = 0, uDetached = 0;
 		for (it = vChans.begin(); it != vChans.end(); ++it) {
-			if (!(*it)->GetName().WildCmp(sChan))
+			CChan *pChannel = *it;
+			CString channelName = pChannel->GetName().AsLower();
+
+			if (!channelName.WildCmp(sChan))
 				continue;
 			uMatches++;
 
@@ -411,7 +419,7 @@ void CClient::UserCommand(CString& sLine) {
 		CIRCSock* pIRCSock = pNetwork->GetIRCSock();
 		const CString& sPerms = (pIRCSock) ? pIRCSock->GetPerms() : "";
 
-		if (!vChans.size()) {
+		if (vChans.empty()) {
 			PutStatus("There are no channels defined.");
 			return;
 		}
@@ -469,12 +477,17 @@ void CClient::UserCommand(CString& sLine) {
 			PutStatus("Usage: AddNetwork <name>");
 			return;
 		}
+		if (!CIRCNetwork::IsValidNetwork(sNetwork)) {
+			PutStatus("Network name should be alphanumeric");
+			return;
+		}
 
-		if (m_pUser->AddNetwork(sNetwork)) {
+		CString sNetworkAddError;
+		if (m_pUser->AddNetwork(sNetwork, sNetworkAddError)) {
 			PutStatus("Network added. Use /znc JumpNetwork " + sNetwork + ", or connect to ZNC with username " + m_pUser->GetUserName() + "/" + sNetwork + " (instead of just " + m_pUser->GetUserName() + ") to connect to it.");
 		} else {
 			PutStatus("Unable to add that network");
-			PutStatus("Perhaps that network is already added");
+			PutStatus(sNetworkAddError);
 		}
 	} else if (sCommand.Equals("DELNETWORK")) {
 		CString sNetwork = sLine.Token(1);
@@ -602,10 +615,11 @@ void CClient::UserCommand(CString& sLine) {
 			fOldNVFile.Copy(sNewModPath + "/.registry");
 		}
 
-		CIRCNetwork* pNewNetwork = pNewUser->AddNetwork(sNewNetwork);
+		CString sNetworkAddError;
+		CIRCNetwork* pNewNetwork = pNewUser->AddNetwork(sNewNetwork, sNetworkAddError);
 
 		if (!pNewNetwork) {
-			PutStatus("Error adding network.");
+			PutStatus("Error adding network:" + sNetworkAddError);
 			return;
 		}
 
@@ -670,7 +684,7 @@ void CClient::UserCommand(CString& sLine) {
 		CString sPass = sLine.Token(3);
 
 		if (sServer.empty()) {
-			PutStatus("Usage: RemServer <host> [port] [pass]");
+			PutStatus("Usage: DelServer <host> [port] [pass]");
 			return;
 		}
 
@@ -828,7 +842,7 @@ void CClient::UserCommand(CString& sLine) {
 		set<CModInfo> ssUserMods;
 		CZNC::Get().GetModules().GetAvailableMods(ssUserMods);
 
-		if (!ssUserMods.size()) {
+		if (ssUserMods.empty()) {
 			PutStatus("No user modules available.");
 		} else {
 			PutStatus("User modules:");
@@ -850,7 +864,7 @@ void CClient::UserCommand(CString& sLine) {
 		set<CModInfo> ssNetworkMods;
 		CZNC::Get().GetModules().GetAvailableMods(ssNetworkMods, CModInfo::NetworkModule);
 
-		if (!ssNetworkMods.size()) {
+		if (ssNetworkMods.empty()) {
 			PutStatus("No network modules available.");
 		} else {
 			PutStatus("Network modules:");
@@ -1106,11 +1120,11 @@ void CClient::UserCommand(CString& sLine) {
 		} else {
 			PutStatus("The host [" + sHost + "] is already in the list");
 		}
-	} else if ((sCommand.Equals("REMBINDHOST") || sCommand.Equals("REMVHOST") || sCommand.Equals("DELVHOST")) && m_pUser->IsAdmin()) {
+	} else if ((sCommand.Equals("REMBINDHOST") || sCommand.Equals("DELBINDHOST") || sCommand.Equals("REMVHOST") || sCommand.Equals("DELVHOST")) && m_pUser->IsAdmin()) {
 		CString sHost = sLine.Token(1);
 
 		if (sHost.empty()) {
-			PutStatus("Usage: RemBindHost <host>");
+			PutStatus("Usage: DelBindHost <host>");
 			return;
 		}
 
@@ -1481,7 +1495,7 @@ void CClient::HelpUser() {
 	Table.AddColumn("Arguments");
 	Table.AddColumn("Description");
 
-	PutStatus("In the following list all occurences of <#chan> support wildcards (* and ?)");
+	PutStatus("In the following list all occurrences of <#chan> support wildcards (* and ?)");
 	PutStatus("(Except ListNicks)");
 
 	Table.AddRow();
@@ -1549,7 +1563,7 @@ void CClient::HelpUser() {
 	Table.SetCell("Description", "Add a server to the list of alternate/backup servers of current IRC network.");
 
 	Table.AddRow();
-	Table.SetCell("Command", "RemServer");
+	Table.SetCell("Command", "DelServer");
 	Table.SetCell("Arguments", "<host> [port] [pass]");
 	Table.SetCell("Description", "Remove a server from the list of alternate/backup servers of current IRC network");
 
@@ -1593,7 +1607,7 @@ void CClient::HelpUser() {
 		Table.SetCell("Description", "Adds a bind host for normal users to use");
 
 		Table.AddRow();
-		Table.SetCell("Command", "RemBindHost");
+		Table.SetCell("Command", "DelBindHost");
 		Table.SetCell("Arguments", "<host>");
 		Table.SetCell("Description", "Removes a bind host from the list");
 	}
