@@ -1,21 +1,26 @@
 /*
- * Copyright (C) 2004-2012  See the AUTHORS file for details.
+ * Copyright (C) 2004-2013 ZNC, see the NOTICE file for details.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 2 as published
- * by the Free Software Foundation.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 #include <znc/IRCNetwork.h>
-#include <znc/Modules.h>
 #include <znc/User.h>
 #include <znc/FileUtils.h>
 #include <znc/Config.h>
-#include <znc/Client.h>
 #include <znc/IRCSock.h>
 #include <znc/Server.h>
 #include <znc/Chan.h>
-#include <znc/znc.h>
 
 using std::vector;
 using std::set;
@@ -312,6 +317,12 @@ bool CIRCNetwork::ParseConfig(CConfig *pConfig, CString& sError, bool bUpgrade)
 						"loading [awaystore] instead");
 				sModName = "awaystore";
 			}
+		
+			// XXX Legacy crap, added in 1.1; fakeonline module was dropped in 1.0 and returned in 1.1
+			if (sModName == "fakeonline") {
+				CUtils::PrintMessage("NOTICE: [fakeonline] was renamed, loading [modules_online] instead");
+				sModName = "modules_online";
+			}
 
 			CUtils::PrintAction("Loading network module [" + sModName + "]");
 			CString sModRet;
@@ -468,7 +479,7 @@ void CIRCNetwork::ClientConnected(CClient *pClient) {
 		}
 
 		const CNick& Nick = GetIRCNick();
-		if (!sClientNick.Equals(Nick.GetNick())) {
+		if (sClientNick != Nick.GetNick()) { // case-sensitive match
 			pClient->PutClient(":" + sClientNick + "!" + Nick.GetIdent() +
 					"@" + Nick.GetHost() + " NICK :" + Nick.GetNick());
 			pClient->SetNick(Nick.GetNick());
@@ -619,7 +630,8 @@ const vector<CChan*>& CIRCNetwork::GetChans() const { return m_vChans; }
 
 CChan* CIRCNetwork::FindChan(CString sName) const {
 	if (GetIRCSock()) {
-		sName.TrimLeft(GetIRCSock()->GetPerms());
+		// See https://tools.ietf.org/html/draft-brocklesby-irc-isupport-03#section-3.16
+		sName.TrimLeft(GetIRCSock()->GetISupport("STATUSMSG", ""));
 	}
 
 	for (unsigned int a = 0; a < m_vChans.size(); a++) {
@@ -671,46 +683,67 @@ bool CIRCNetwork::DelChan(const CString& sName) {
 }
 
 void CIRCNetwork::JoinChans() {
+	// Avoid divsion by zero, it's bad!
+	if (m_vChans.empty())
+		return;
+
+	// We start at a random offset into the channel list so that if your
+	// first 3 channels are invite-only and you got MaxJoins == 3, ZNC will
+	// still be able to join the rest of your channels.
+	unsigned int start = rand() % m_vChans.size();
+	unsigned int uJoins = m_pUser->MaxJoins();
+	set<CChan*> sChans;
+	for (unsigned int a = 0; a < m_vChans.size(); a++) {
+		unsigned int idx = (start + a) % m_vChans.size();
+		CChan* pChan = m_vChans[idx];
+		if (!pChan->IsOn() && !pChan->IsDisabled()) {
+			if (!JoinChan(pChan))
+				continue;
+
+			sChans.insert(pChan);
+
+			// Limit the number of joins
+			if (uJoins != 0 && --uJoins == 0)
+				break;
+		}
+	}
+
+	while (!sChans.empty())
+		JoinChans(sChans);
+}
+
+void CIRCNetwork::JoinChans(set<CChan*>& sChans) {
+	CString sKeys, sJoin;
 	bool bHaveKey = false;
-	size_t joinLength = 4;  // join
-	CString sChannels, sKeys;
+	size_t uiJoinLength = strlen("JOIN ");
 
-	for (vector<CChan*>::iterator it = m_vChans.begin(); it != m_vChans.end(); ++it) {
-		CChan *pChan = *it;
+	while (!sChans.empty()) {
+		set<CChan*>::iterator it = sChans.begin();
+		const CString& sName = (*it)->GetName();
+		const CString& sKey = (*it)->GetKey();
+		size_t len = sName.length() + sKey.length();
+		len += 2; // two comma
 
-		if (pChan->IsOn() || pChan->IsDisabled() || !JoinChan(pChan)) {
-			continue;
-		}
+		if (!sKeys.empty() && uiJoinLength + len >= 512)
+			break;
 
-		size_t length = pChan->GetName().length() + pChan->GetKey().length() + 2;  // +2 for either space or commas
-
-		if ((joinLength + length) >= 510) {
-			// Sent what we got, and cleanup
-			PutIRC("JOIN " + sChannels + (bHaveKey ? (" " + sKeys) : ""));
-
-			sChannels = "";
-			sKeys = "";
-			joinLength = 4;  // join
-			bHaveKey = false;
-		}
-
-		if (!sChannels.empty()) {
-			sChannels += ",";
+		if (!sJoin.empty()) {
+			sJoin += ",";
 			sKeys += ",";
 		}
-
-		if (!pChan->GetKey().empty()) {
+		uiJoinLength += len;
+		sJoin += sName;
+		if (!sKey.empty()) {
+			sKeys += sKey;
 			bHaveKey = true;
-			sKeys += pChan->GetKey();
 		}
-
-		sChannels += pChan->GetName();
-		joinLength += length;
+		sChans.erase(it);
 	}
 
-	if (!sChannels.empty()) {
-		PutIRC("JOIN " + sChannels + (bHaveKey ? (" " + sKeys) : ""));
-	}
+	if (bHaveKey)
+		PutIRC("JOIN " + sJoin + " " + sKeys);
+	else
+		PutIRC("JOIN " + sJoin);
 }
 
 bool CIRCNetwork::JoinChan(CChan* pChan) {
@@ -760,7 +793,7 @@ bool CIRCNetwork::DelServer(const CString& sName, unsigned short uPort, const CS
 	bool bSawCurrentServer = false;
 	CServer* pCurServer = GetCurrentServer();
 
-	for (vector<CServer*>::iterator it = m_vServers.begin(); it != m_vServers.end(); it++, a++) {
+	for (vector<CServer*>::iterator it = m_vServers.begin(); it != m_vServers.end(); ++it, a++) {
 		CServer* pServer = *it;
 
 		if (pServer == pCurServer)
