@@ -1,17 +1,23 @@
 /*
- * Copyright (C) 2004-2012  See the AUTHORS file for details.
+ * Copyright (C) 2004-2013 ZNC, see the NOTICE file for details.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 2 as published
- * by the Free Software Foundation.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 #include <znc/IRCSock.h>
 #include <znc/Chan.h>
-#include <znc/Client.h>
 #include <znc/User.h>
 #include <znc/IRCNetwork.h>
-#include <znc/znc.h>
 #include <znc/Server.h>
 
 using std::set;
@@ -242,6 +248,9 @@ void CIRCSock::ReadLine(const CString& sData) {
 						// reply isn't forwarded.
 						return;
 					}
+					if (pChan->IsDetached()) {
+						return;
+					}
 				}
 			}
 				break;
@@ -260,6 +269,9 @@ void CIRCSock::ReadLine(const CString& sData) {
 						// reply isn't forwarded.
 						return;
 					}
+					if (pChan->IsDetached()) {
+						return;
+					}
 				}
 			}
 				break;
@@ -269,6 +281,9 @@ void CIRCSock::ReadLine(const CString& sData) {
 
 				if (pChan) {
 					pChan->SetTopic("");
+					if (pChan->IsDetached()) {
+						return;
+					}
 				}
 
 				break;
@@ -281,6 +296,9 @@ void CIRCSock::ReadLine(const CString& sData) {
 					CString sTopic = sLine.Token(4, true);
 					sTopic.LeftChomp();
 					pChan->SetTopic(sTopic);
+					if (pChan->IsDetached()) {
+						return;
+					}
 				}
 
 				break;
@@ -295,6 +313,10 @@ void CIRCSock::ReadLine(const CString& sData) {
 
 					pChan->SetTopicOwner(sNick);
 					pChan->SetTopicDate(ulDate);
+
+					if (pChan->IsDetached()) {
+						return;
+					}
 				}
 
 				break;
@@ -303,6 +325,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 				// :irc.yourserver.com 352 yournick #chan ident theirhost.com irc.theirserver.com theirnick H :0 Real Name
 				sServer = sLine.Token(0);
 				sNick = sLine.Token(7);
+				CString sChan = sLine.Token(3);
 				CString sIdent = sLine.Token(4);
 				CString sHost = sLine.Token(5);
 
@@ -342,7 +365,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 								sNewNick = sNick[0] + sNick.substr(pos);
 							}
 							CString sNewLine = sServer + " 352 " + sLine.Token(2) + " " +
-								sLine.Token(3) + " " + sIdent + " " + sHost + " " +
+								sChan + " " + sIdent + " " + sHost + " " +
 								sLine.Token(6)  + " " + sNewNick + " " +
 								sLine.Token(8, true);
 							m_pNetwork->PutUser(sNewLine, pClient);
@@ -352,6 +375,11 @@ void CIRCSock::ReadLine(const CString& sData) {
 					return;
 				}
 
+				CChan* pChan = m_pNetwork->FindChan(sChan);
+				if (pChan && pChan->IsDetached()) {
+					return;
+				}
+
 				break;
 			}
 			case 353: {  // NAMES
@@ -363,6 +391,9 @@ void CIRCSock::ReadLine(const CString& sData) {
 				if (pChan) {
 					CString sNicks = sRest.Token(2, true).TrimPrefix_n();
 					pChan->AddNicks(sNicks);
+					if (pChan->IsDetached()) {
+						return;
+					}
 				}
 
 				ForwardRaw353(sLine);
@@ -371,8 +402,6 @@ void CIRCSock::ReadLine(const CString& sData) {
 				return;
 			}
 			case 366: {  // end of names list
-				m_pNetwork->PutUser(sLine);  // First send them the raw
-
 				// :irc.server.com 366 nick #chan :End of /NAMES list.
 				CChan* pChan = m_pNetwork->FindChan(sRest.Token(0));
 
@@ -391,13 +420,14 @@ void CIRCSock::ReadLine(const CString& sData) {
 							}
 						}
 					}
+					if (pChan->IsDetached()) {
+						// don't put it to clients
+						return;
+					}
 				}
 
-				return;  // return so we don't send them the raw twice
-			}
-			case 381:  // You are now an IRC Operator
-				m_pNetwork->AddRawBuffer(":" + _NAMEDFMT(sServer) + " " + sCmd + " {target} " + _NAMEDFMT(sRest));
 				break;
+			}
 			case 375:  // begin motd
 			case 422:  // MOTD File is missing
 				if (m_pNetwork->GetIRCServer().Equals(sServer)) {
@@ -482,8 +512,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 				}
 			}
 
-			// Todo: use nick compare function here
-			if (Nick.GetNick().Equals(GetNick())) {
+			if (Nick.NickEquals(GetNick())) {
 				// We are changing our own nick, the clients always must see this!
 				bIsVisible = false;
 				SetNick(sNewNick);
@@ -501,7 +530,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 
 			// :nick!ident@host.com QUIT :message
 
-			if (Nick.GetNick().Equals(GetNick())) {
+			if (Nick.NickEquals(GetNick())) {
 				m_pNetwork->PutStatus("You quit [" + sMessage + "]");
 				// We don't call module hooks and we don't
 				// forward this quit to clients (Some clients
@@ -533,8 +562,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 			CString sChan = sRest.Token(0).TrimPrefix_n();
 			CChan* pChan;
 
-			// Todo: use nick compare function
-			if (Nick.GetNick().Equals(GetNick())) {
+			if (Nick.NickEquals(GetNick())) {
 				m_pNetwork->AddChan(sChan, false);
 				pChan = m_pNetwork->FindChan(sChan);
 				if (pChan) {
@@ -568,8 +596,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 					bDetached = true;
 			}
 
-			// Todo: use nick compare function
-			if (Nick.GetNick().Equals(GetNick())) {
+			if (Nick.NickEquals(GetNick())) {
 				m_pNetwork->DelChan(sChan);
 			}
 
@@ -673,7 +700,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 				}
 			}
 
-			if (Nick.GetNick().Equals(m_pNetwork->GetIRCServer())) {
+			if (Nick.NickEquals(m_pNetwork->GetIRCServer())) {
 				m_pNetwork->PutUser(":" + Nick.GetNick() + " NOTICE " + sTarget + " :" + sMsg);
 			} else {
 				m_pNetwork->PutUser(":" + Nick.GetNickMask() + " NOTICE " + sTarget + " :" + sMsg);
@@ -991,19 +1018,26 @@ bool CIRCSock::OnChanMsg(CNick& Nick, const CString& sChan, CString& sMessage) {
 }
 
 void CIRCSock::PutIRC(const CString& sLine) {
-	DEBUG("(" << m_pNetwork->GetUser()->GetUserName() << "/" << m_pNetwork->GetName() << ") ZNC -> IRC [" << sLine << "] (queued)");
+	// Only print if the line won't get sent immediately (same condition as in TrySend()!)
+	if (m_bFloodProtection && m_iSendsAllowed <= 0) {
+		DEBUG("(" << m_pNetwork->GetUser()->GetUserName() << "/" << m_pNetwork->GetName() << ") ZNC -> IRC [" << sLine << "] (queued)");
+	}
 	m_vsSendQueue.push_back(sLine);
 	TrySend();
 }
 
 void CIRCSock::PutIRCQuick(const CString& sLine) {
-	DEBUG("(" << m_pNetwork->GetUser()->GetUserName() << "/" << m_pNetwork->GetName() << ") ZNC -> IRC [" << sLine << "] (queued to front)");
+	// Only print if the line won't get sent immediately (same condition as in TrySend()!)
+	if (m_bFloodProtection && m_iSendsAllowed <= 0) {
+		DEBUG("(" << m_pNetwork->GetUser()->GetUserName() << "/" << m_pNetwork->GetName() << ") ZNC -> IRC [" << sLine << "] (queued to front)");
+	}
 	m_vsSendQueue.push_front(sLine);
 	TrySend();
 }
 
 void CIRCSock::TrySend() {
-	while (!m_vsSendQueue.empty() && (m_iSendsAllowed > 0 || !m_bFloodProtection)) {
+	// This condition must be the same as in PutIRC() and PutIRCQuick()!
+	while (!m_vsSendQueue.empty() && (!m_bFloodProtection || m_iSendsAllowed > 0)) {
 		m_iSendsAllowed--;
 		DEBUG("(" << m_pNetwork->GetUser()->GetUserName() << "/" << m_pNetwork->GetName() << ") ZNC -> IRC [" << m_vsSendQueue.front() << "]");
 		Write(m_vsSendQueue.front() + "\r\n");
@@ -1097,7 +1131,7 @@ void CIRCSock::Timeout() {
 	m_pNetwork->ClearMotdBuffer();
 
 	ResetChans();
-	m_scUserModes.empty();
+	m_scUserModes.clear();
 }
 
 void CIRCSock::ConnectionRefused() {
@@ -1174,6 +1208,15 @@ void CIRCSock::ParseISupport(const CString& sLine) {
 	}
 }
 
+CString CIRCSock::GetISupport(const CString& sKey, const CString& sDefault) const {
+	MCString::const_iterator i = m_mISupport.find(sKey.AsUpper());
+	if (i == m_mISupport.end()) {
+		return sDefault;
+	} else {
+		return i->second;
+	}
+}
+
 void CIRCSock::ForwardRaw353(const CString& sLine) const {
 	vector<CClient*>& vClients = m_pNetwork->GetClients();
 	vector<CClient*>::iterator it;
