@@ -1,9 +1,17 @@
 /*
- * Copyright (C) 2004-2012  See the AUTHORS file for details.
+ * Copyright (C) 2004-2013 ZNC, see the NOTICE file for details.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 2 as published
- * by the Free Software Foundation.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 #include <znc/Modules.h>
@@ -48,7 +56,7 @@ bool ZNC_NO_NEED_TO_DO_ANYTHING_ON_MODULE_CALL_EXITER;
 			if (m_pNetwork)                              \
 				pMod->SetNetwork(pNetwork);              \
 			pMod->SetClient(pOldClient);                     \
-		} catch (CModule::EModException e) {                     \
+		} catch (const CModule::EModException& e) {                     \
 			if (e == CModule::UNLOAD) {                      \
 				UnloadModule((*this)[a]->GetModName());  \
 			}                                                \
@@ -88,7 +96,7 @@ bool ZNC_NO_NEED_TO_DO_ANYTHING_ON_MODULE_CALL_EXITER;
 				bHaltCore = true;                        \
 				break;                                   \
 			}                                                \
-		} catch (CModule::EModException e) {                     \
+		} catch (const CModule::EModException& e) {                     \
 			if (e == CModule::UNLOAD) {                      \
 				UnloadModule((*this)[a]->GetModName());  \
 			}                                                \
@@ -155,6 +163,25 @@ void CModule::SetUser(CUser* pUser) { m_pUser = pUser; }
 void CModule::SetNetwork(CIRCNetwork* pNetwork) { m_pNetwork = pNetwork; }
 void CModule::SetClient(CClient* pClient) { m_pClient = pClient; }
 
+CString CModule::ExpandString(const CString& sStr) const {
+	CString sRet;
+	return ExpandString(sStr, sRet);
+}
+
+CString& CModule::ExpandString(const CString& sStr, CString& sRet) const {
+	sRet = sStr;
+
+	if (m_pNetwork) {
+		return m_pNetwork->ExpandString(sRet, sRet);
+	}
+
+	if (m_pUser) {
+		return m_pUser->ExpandString(sRet, sRet);
+	}
+
+	return sRet;
+}
+
 const CString& CModule::GetSavePath() const {
 	if (!CFile::Exists(m_sSavePath)) {
 		CDir::MakeDir(m_sSavePath);
@@ -573,6 +600,8 @@ CModule::EModRet CModule::OnPrivNotice(CNick& Nick, CString& sMessage) { return
 CModule::EModRet CModule::OnChanNotice(CNick& Nick, CChan& Channel, CString& sMessage) { return CONTINUE; }
 CModule::EModRet CModule::OnTopic(CNick& Nick, CChan& Channel, CString& sTopic) { return CONTINUE; }
 CModule::EModRet CModule::OnTimerAutoJoin(CChan& Channel) { return CONTINUE; }
+CModule::EModRet CModule::OnAddNetwork(CIRCNetwork& Network, CString& sErrorRet) { return CONTINUE; }
+CModule::EModRet CModule::OnDeleteNetwork(CIRCNetwork& Network) { return CONTINUE; }
 
 bool CModule::OnServerCapAvailable(const CString& sCap) { return false; }
 void CModule::OnServerCapResult(const CString& sCap, bool bSuccess) {}
@@ -670,7 +699,7 @@ bool CModules::OnBoot() {
 			if (!(*this)[a]->OnBoot()) {
 				return true;
 			}
-		} catch (CModule::EModException e) {
+		} catch (const CModule::EModException& e) {
 			if (e == CModule::UNLOAD) {
 				UnloadModule((*this)[a]->GetModName());
 			}
@@ -731,6 +760,8 @@ bool CModules::OnPrivNotice(CNick& Nick, CString& sMessage) { MODHALTCHK(OnPrivN
 bool CModules::OnChanNotice(CNick& Nick, CChan& Channel, CString& sMessage) { MODHALTCHK(OnChanNotice(Nick, Channel, sMessage)); }
 bool CModules::OnTopic(CNick& Nick, CChan& Channel, CString& sTopic) { MODHALTCHK(OnTopic(Nick, Channel, sTopic)); }
 bool CModules::OnTimerAutoJoin(CChan& Channel) { MODHALTCHK(OnTimerAutoJoin(Channel)); }
+bool CModules::OnAddNetwork(CIRCNetwork& Network, CString& sErrorRet) { MODHALTCHK(OnAddNetwork(Network, sErrorRet)); }
+bool CModules::OnDeleteNetwork(CIRCNetwork& Network) { MODHALTCHK(OnDeleteNetwork(Network)); }
 bool CModules::OnStatusCommand(CString& sCommand) { MODHALTCHK(OnStatusCommand(sCommand)); }
 bool CModules::OnModCommand(const CString& sCommand) { MODUNLOADCHK(OnModCommand(sCommand)); return false; }
 bool CModules::OnModNotice(const CString& sMessage) { MODUNLOADCHK(OnModNotice(sMessage)); return false; }
@@ -754,7 +785,7 @@ bool CModules::OnServerCapAvailable(const CString& sCap) {
 				bResult |= pMod->OnServerCapAvailable(sCap);
 			}
 			pMod->SetClient(pOldClient);
-		} catch (CModule::EModException e) {
+		} catch (const CModule::EModException& e) {
 			if (CModule::UNLOAD == e) {
 				UnloadModule((*this)[a]->GetModName());
 			}
@@ -817,7 +848,7 @@ bool CModules::IsClientCapSupported(CClient* pClient, const CString& sCap, bool
 				bResult |= pMod->IsClientCapSupported(pClient, sCap, bState);
 			}
 			pMod->SetClient(pOldClient);
-		} catch (CModule::EModException e) {
+		} catch (const CModule::EModException& e) {
 			if (CModule::UNLOAD == e) {
 				UnloadModule((*this)[a]->GetModName());
 			}
@@ -923,7 +954,7 @@ bool CModules::LoadModule(const CString& sModule, const CString& sArgs, CModInfo
 	bool bLoaded;
 	try {
 		bLoaded = pModule->OnLoad(sArgs, sRetMsg);
-	} catch (CModule::EModException) {
+	} catch (const CModule::EModException&) {
 		bLoaded = false;
 		sRetMsg = "Caught an exception";
 	}
@@ -1149,7 +1180,11 @@ ModHandle CModules::OpenModule(const CString& sModule, const CString& sModPath,
 	ModHandle p = dlopen((sModPath).c_str(), RTLD_NOW | RTLD_GLOBAL);
 
 	if (!p) {
-		sRetMsg = "Unable to open module [" + sModule + "] [" + dlerror() + "]";
+		// dlerror() returns pointer to static buffer, which may be overwritten very soon with another dl call
+		// also it may just return null.
+		const char* cDlError = dlerror();
+		CString sDlError = cDlError ? cDlError : "Unknown error";
+		sRetMsg = "Unable to open module [" + sModule + "] [" + sDlError + "]";
 		return NULL;
 	}
 
