@@ -1,20 +1,24 @@
 /*
- * Copyright (C) 2004-2012  See the AUTHORS file for details.
+ * Copyright (C) 2004-2013 ZNC, see the NOTICE file for details.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 2 as published
- * by the Free Software Foundation.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 #include <znc/User.h>
-#include <znc/Chan.h>
 #include <znc/Config.h>
 #include <znc/FileUtils.h>
 #include <znc/IRCNetwork.h>
 #include <znc/IRCSock.h>
-#include <znc/Server.h>
-#include <znc/znc.h>
-#include <znc/Modules.h>
 #include <math.h>
 
 using std::vector;
@@ -55,6 +59,15 @@ protected:
 				}
 			}
 		}
+
+		vector<CClient*>& vUserClients = m_pUser->GetUserClients();
+		for (size_t c = 0; c < vUserClients.size(); ++c) {
+			CClient* pUserClient = vUserClients[c];
+
+			if (pUserClient->GetTimeSinceLastDataTransaction() >= 270) {
+				pUserClient->PutClient("PING :ZNC");
+			}
+		}
 	}
 
 	CUser* m_pUser;
@@ -82,6 +95,7 @@ CUser::CUser(const CString& sUserName)
 	m_uBufferCount = 50;
 	m_uMaxJoinTries = 10;
 	m_bAutoClearChanBuffer = true;
+	m_uMaxJoins = 0;
 	m_bBeingDeleted = false;
 	m_sTimestampFormat = "[%H:%M:%S]";
 	m_bAppendTimestamp = false;
@@ -98,9 +112,8 @@ CUser::~CUser() {
 	}
 
 	// Delete clients
-	for (unsigned int c = 0; c < m_vClients.size(); c++) {
-		CClient* pClient = m_vClients[c];
-		CZNC::Get().GetManager().DelSockByAddr(pClient);
+	while (!m_vClients.empty()) {
+		CZNC::Get().GetManager().DelSockByAddr(m_vClients[0]);
 	}
 	m_vClients.clear();
 
@@ -139,6 +152,7 @@ bool CUser::ParseConfig(CConfig* pConfig, CString& sError) {
 	TOption<unsigned int> UIntOptions[] = {
 		{ "jointries", &CUser::SetJoinTries },
 		{ "maxnetworks", &CUser::SetMaxNetworks },
+		{ "maxjoins", &CUser::SetMaxJoins },
 	};
 	size_t numUIntOptions = sizeof(UIntOptions) / sizeof(UIntOptions[0]);
 	TOption<bool> BoolOptions[] = {
@@ -184,9 +198,6 @@ bool CUser::ParseConfig(CConfig* pConfig, CString& sError) {
 
 	CString sValue;
 
-	// MaxJoins has been removed
-	pConfig->FindStringEntry("maxjoins", sValue);
-
 	CString sDCCLookupValue;
 	pConfig->FindStringEntry("dcclookupmethod", sDCCLookupValue);
 	if (pConfig->FindStringEntry("bouncedccs", sValue))  {
@@ -209,7 +220,7 @@ bool CUser::ParseConfig(CConfig* pConfig, CString& sError) {
 	if (pConfig->FindStringEntry("buffer", sValue))
 		SetBufferCount(sValue.ToUInt(), true);
 	if (pConfig->FindStringEntry("awaysuffix", sValue)) {
-		CUtils::PrintMessage("WARNING: AwaySuffix has been depricated, instead try -> LoadModule = awaynick %nick%_" + sValue);
+		CUtils::PrintMessage("WARNING: AwaySuffix has been deprecated, instead try -> LoadModule = awaynick %nick%_" + sValue);
 	}
 	if (pConfig->FindStringEntry("autocycle", sValue)) {
 		if (sValue.Equals("true"))
@@ -318,7 +329,7 @@ bool CUser::ParseConfig(CConfig* pConfig, CString& sError) {
 			CZNC::DumpConfig(pSubConf);
 			return false;
 		}
-		subIt++;
+		++subIt;
 	}
 	if (subIt != subConf.end()) {
 		sError = "Password defined more than once";
@@ -346,7 +357,8 @@ bool CUser::ParseConfig(CConfig* pConfig, CString& sError) {
 	if (pConfig->FindStringVector("server", vsList, false) || pConfig->FindStringVector("chan", vsList, false) || pConfig->FindSubConfig("chan", subConf, false)) {
 		CIRCNetwork *pNetwork = FindNetwork("default");
 		if (!pNetwork) {
-			pNetwork = AddNetwork("default");
+			CString sErrorDummy;
+			pNetwork = AddNetwork("default", sErrorDummy);
 		}
 
 		if (pNetwork) {
@@ -380,6 +392,19 @@ bool CUser::ParseConfig(CConfig* pConfig, CString& sError) {
 			CUtils::PrintMessage("NOTICE: [admin] module was renamed, loading [controlpanel] instead");
 			sModName = "controlpanel";
 		}
+		
+		// XXX Legacy crap, should have been added ZNC 0.207, but added only in 1.1 :(
+		if (sModName == "away") {
+			CUtils::PrintMessage("NOTICE: [away] was renamed, "
+					"loading [awaystore] instead");
+			sModName = "awaystore";
+		}
+
+		// XXX Legacy crap, added in 1.1; fakeonline module was dropped in 1.0 and returned in 1.1
+		if (sModName == "fakeonline") {
+			CUtils::PrintMessage("NOTICE: [fakeonline] was renamed, loading [modules_online] instead");
+			sModName = "modules_online";
+		}
 
 		CUtils::PrintAction("Loading user module [" + sModName + "]");
 		CString sModRet;
@@ -435,12 +460,26 @@ bool CUser::ParseConfig(CConfig* pConfig, CString& sError) {
 	return true;
 }
 
-CIRCNetwork* CUser::AddNetwork(const CString &sNetwork) {
-	if (!CIRCNetwork::IsValidNetwork(sNetwork) || FindNetwork(sNetwork)) {
+CIRCNetwork* CUser::AddNetwork(const CString &sNetwork, CString& sErrorRet) {
+	if (!CIRCNetwork::IsValidNetwork(sNetwork)) {
+		sErrorRet = "Invalid network name. It should be alphanumeric. Not to be confused with server name";
+		return NULL;
+	} else if (FindNetwork(sNetwork)) {
+		sErrorRet = "Network [" + sNetwork.Token(0) + "] already exists";
 		return NULL;
 	}
 
-	return new CIRCNetwork(this, sNetwork);
+	CIRCNetwork* pNetwork = new CIRCNetwork(this, sNetwork);
+
+	bool bCancel = false;
+	USERMODULECALL(OnAddNetwork(*pNetwork, sErrorRet), this, NULL, &bCancel);
+	if(bCancel) {
+		RemoveNetwork(pNetwork);
+		delete pNetwork;
+		return NULL;
+	}
+
+	return pNetwork;
 }
 
 bool CUser::AddNetwork(CIRCNetwork *pNetwork) {
@@ -466,8 +505,12 @@ bool CUser::DeleteNetwork(const CString& sNetwork) {
 	CIRCNetwork *pNetwork = FindNetwork(sNetwork);
 
 	if (pNetwork) {
-		delete pNetwork;
-		return true;
+		bool bCancel = false;
+		USERMODULECALL(OnDeleteNetwork(*pNetwork), this, NULL, &bCancel);
+		if (!bCancel) {
+			delete pNetwork;
+			return true;
+		}
 	}
 
 	return false;
@@ -656,6 +699,7 @@ bool CUser::Clone(const CUser& User, CString& sErrorRet, bool bCloneNetworks) {
 	SetBufferCount(User.GetBufferCount(), true);
 	SetJoinTries(User.JoinTries());
 	SetMaxNetworks(User.MaxNetworks());
+	SetMaxJoins(User.MaxJoins());
 
 	// Allowed Hosts
 	m_ssAllowedHosts.clear();
@@ -851,6 +895,7 @@ CConfig CUser::ToConfig() {
 	config.AddKeyValuePair("Timezone", m_sTimezone);
 	config.AddKeyValuePair("JoinTries", CString(m_uMaxJoinTries));
 	config.AddKeyValuePair("MaxNetworks", CString(m_uMaxNetworks));
+	config.AddKeyValuePair("MaxJoins", CString(m_uMaxJoins));
 
 	// Allow Hosts
 	if (!m_ssAllowedHosts.empty()) {
