@@ -1,23 +1,25 @@
 /*
- * Copyright (C) 2004-2012  See the AUTHORS file for details.
+ * Copyright (C) 2004-2013 ZNC, see the NOTICE file for details.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 2 as published
- * by the Free Software Foundation.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 #include <znc/Utils.h>
-#include <znc/MD5.h>
-#include <znc/main.h>
 #include <znc/ZNCDebug.h>
 #include <znc/FileUtils.h>
-#include <errno.h>
 #ifdef HAVE_LIBSSL
 #include <openssl/ssl.h>
 #endif /* HAVE_LIBSSL */
-#include <sstream>
-#include <sys/stat.h>
-#include <sys/types.h>
 #include <unistd.h>
 
 // Required with GCC 4.3+ if openssl is disabled
@@ -343,13 +345,37 @@ void CUtils::PrintStatus(bool bSuccess, const CString& sMessage) {
 	fflush(stdout);
 }
 
-CString CUtils::CTime(time_t t, const CString& sTZ) {
+namespace {
+	/* Switch GMT-X and GMT+X
+	 *
+	 * See https://en.wikipedia.org/wiki/Tz_database#Area
+	 *
+	 * "In order to conform with the POSIX style, those zone names beginning
+	 * with "Etc/GMT" have their sign reversed from what most people expect.
+	 * In this style, zones west of GMT have a positive sign and those east
+	 * have a negative sign in their name (e.g "Etc/GMT-14" is 14 hours
+	 * ahead/east of GMT.)"
+	 */
+	inline CString FixGMT(CString sTZ) {
+		if (sTZ.length() >= 4 && sTZ.Left(3) == "GMT") {
+			if (sTZ[3] == '+') {
+				sTZ[3] = '-';
+			} else if (sTZ[3] == '-') {
+				sTZ[3] = '+';
+			}
+		}
+		return sTZ;
+	}
+}
+
+CString CUtils::CTime(time_t t, const CString& sTimezone) {
 	char s[30] = {}; // should have at least 26 bytes
-	if (sTZ.empty()) {
+	if (sTimezone.empty()) {
 		ctime_r(&t, s);
 		// ctime() adds a trailing newline
 		return CString(s).Trim_n();
 	}
+	CString sTZ = FixGMT(sTimezone);
 
 	// backup old value
 	char* oldTZ = getenv("TZ");
@@ -371,14 +397,15 @@ CString CUtils::CTime(time_t t, const CString& sTZ) {
 	return CString(s).Trim_n();
 }
 
-CString CUtils::FormatTime(time_t t, const CString& sFormat, const CString& sTZ) {
+CString CUtils::FormatTime(time_t t, const CString& sFormat, const CString& sTimezone) {
 	char s[1024] = {};
 	tm m;
-	if (sTZ.empty()) {
+	if (sTimezone.empty()) {
 		localtime_r(&t, &m);
 		strftime(s, sizeof(s), sFormat.c_str(), &m);
 		return s;
 	}
+	CString sTZ = FixGMT(sTimezone);
 
 	// backup old value
 	char* oldTZ = getenv("TZ");
