@@ -1,9 +1,17 @@
 /*
- * Copyright (C) 2004-2012  See the AUTHORS file for details.
+ * Copyright (C) 2004-2013 ZNC, see the NOTICE file for details.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 2 as published
- * by the Free Software Foundation.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 #include <znc/WebModules.h>
@@ -361,12 +369,32 @@ bool CWebSock::AddModLoop(const CString& sLoopName, CModule& Module, CTemplate *
 
 	if (!sTitle.empty() && (IsLoggedIn() || (!Module.WebRequiresLogin() && !Module.WebRequiresAdmin())) && (GetSession()->IsAdmin() || !Module.WebRequiresAdmin())) {
 		CTemplate& Row = pTemplate->AddRow(sLoopName);
+		bool bActiveModule = false;
 
 		Row["ModName"] = Module.GetModName();
 		Row["ModPath"] = Module.GetWebPath();
 		Row["Title"] = sTitle;
 
 		if (m_sModName == Module.GetModName()) {
+			CString sModuleType = GetPath().Token(1, false, "/");
+			if (sModuleType == "global" && Module.GetType() == CModInfo::GlobalModule) {
+				bActiveModule = true;
+			} else if (sModuleType == "user" && Module.GetType() == CModInfo::UserModule) {
+				bActiveModule = true;
+			} else if (sModuleType == "network" && Module.GetType() == CModInfo::NetworkModule) {
+				CIRCNetwork *Network = Module.GetNetwork();
+				if (Network) {
+					CString sNetworkName = GetPath().Token(2, false, "/");
+					if (sNetworkName == Network->GetName()) {
+						bActiveModule = true;
+					}
+				} else {
+					bActiveModule = true;
+				}
+			}
+		}
+
+		if (bActiveModule) {
 			Row["Active"] = "true";
 		}
 
@@ -380,7 +408,7 @@ bool CWebSock::AddModLoop(const CString& sLoopName, CModule& Module, CTemplate *
 			TWebSubPage& SubPage = vSubPages[a];
 
 			// bActive is whether or not the current url matches this subpage (params will be checked below)
-			bool bActive = (m_sModName == Module.GetModName() && m_sPage == SubPage->GetName());
+			bool bActive = (m_sModName == Module.GetModName() && m_sPage == SubPage->GetName() && bActiveModule);
 
 			if (SubPage->RequiresAdmin() && !GetSession()->IsAdmin()) {
 				continue;  // Don't add admin-only subpages to requests from non-admin users
