@@ -1,12 +1,19 @@
 /*
- * Copyright (C) 2004-2012  See the AUTHORS file for details.
+ * Copyright (C) 2004-2013 ZNC, see the NOTICE file for details.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 2 as published
- * by the Free Software Foundation.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
-#include <znc/ZNCString.h>
 #include <znc/FileUtils.h>
 #include <znc/Utils.h>
 #include <znc/MD5.h>
@@ -165,6 +172,8 @@ CString::EEscape CString::ToEscape(const CString& sEsc) {
 		return ESQL;
 	} else if (sEsc.Equals("NAMEDFMT")) {
 		return ENAMEDFMT;
+	} else if (sEsc.Equals("DEBUG")) {
+		return EDEBUG;
 	}
 
 	return EASCII;
@@ -281,6 +290,30 @@ CString CString::Escape_n(EEscape eFrom, EEscape eTo) const {
 				}
 
 				break;
+			case EDEBUG:
+				if (*p == '\\' && (a +3) < iLength && *(p +1) == 'x' && isxdigit(*(p +2)) && isxdigit(*(p +3))) {
+					p += 2;
+					if (isdigit(*p)) {
+						ch = (unsigned char)((*p - '0') << 4);
+					} else {
+						ch = (unsigned char)((tolower(*p) - 'a' +10) << 4);
+					}
+
+					p++;
+					if (isdigit(*p)) {
+						ch |= (unsigned char)(*p - '0');
+					} else {
+						ch |= (unsigned char)(tolower(*p) - 'a' +10);
+					}
+
+					a += 3;
+				} else if (*p == '\\' && a+1 < iLength && *(p+1) == '.') {
+					a++;
+					p++;
+					ch = '\\';
+				} else {
+					ch = *p;
+				}
 		}
 
 		switch (eTo) {
@@ -327,6 +360,18 @@ CString CString::Escape_n(EEscape eFrom, EEscape eTo) const {
 				} else if (ch == '}') { sRet += '\\'; sRet += '}';
 				} else { sRet += ch; }
 
+				break;
+			case EDEBUG:
+				if (ch < 0x20 || ch == 0x7F) {
+					sRet += "\\x";
+					sRet += szHex[ch >> 4];
+					sRet += szHex[ch & 0xf];
+				} else if (ch == '\\') {
+					sRet += "\\.";
+				} else {
+					sRet += ch;
+				}
+
 				break;
 		}
 	}
@@ -1094,6 +1139,58 @@ bool CString::RightChomp(size_type uLen) {
 	return bRet;
 }
 
+CString CString::StripControls_n() const {
+	CString sRet;
+	const unsigned char *pStart = (const unsigned char*) data();
+	unsigned char ch = *pStart;
+	size_type iLength = length();
+	sRet.reserve(iLength);
+	bool colorCode = false;
+	unsigned int digits = 0;
+	bool comma = false;
+
+	for (unsigned int a = 0; a < iLength; a++, ch = pStart[a]) {
+		// Color code. Format: \x03([0-9]{1,2}(,[0-9]{1,2})?)?
+		if (ch == 0x03) {
+			colorCode = true;
+			digits = 0;
+			comma = false;
+			continue;
+		}
+		if (colorCode) {
+			if (isdigit(ch) && digits < 2) {
+				digits++;
+				continue;
+			}
+			if (ch == ',' && !comma && digits > 0) {
+				comma = true;
+				digits = 0;
+				continue;
+			}
+
+			colorCode = false;
+
+			if (digits == 0 && comma) { // There was a ',' which wasn't followed by digits, we should print it.
+				sRet += ',';
+			}
+		}
+		// CO controls codes
+		if (ch < 0x20 || ch == 0x7F)
+			continue;
+		sRet += ch;
+	}
+	if (colorCode && digits == 0 && comma) {
+		sRet += ',';
+	}
+
+	sRet.reserve(0);
+	return sRet;
+}
+
+CString& CString::StripControls() {
+	return (*this = StripControls_n());
+}
+
 //////////////// MCString ////////////////
 const MCString MCString::EmptyMap;
 
