@@ -1,19 +1,24 @@
 /*
- * Copyright (C) 2004-2012  See the AUTHORS file for details.
+ * Copyright (C) 2004-2013 ZNC, see the NOTICE file for details.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 2 as published
- * by the Free Software Foundation.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
-#include <znc/znc.h>
-#include <znc/Chan.h>
 #include <znc/FileUtils.h>
 #include <znc/IRCSock.h>
 #include <znc/Server.h>
 #include <znc/User.h>
 #include <znc/IRCNetwork.h>
-#include <znc/Listener.h>
 #include <znc/Config.h>
 
 using std::endl;
@@ -54,6 +59,11 @@ CZNC::~CZNC() {
 
 	for (map<CString,CUser*>::iterator a = m_msUsers.begin(); a != m_msUsers.end(); ++a) {
 		a->second->GetModules().UnloadAll();
+
+		const vector<CIRCNetwork*>& networks = a->second->GetNetworks();
+		for (vector<CIRCNetwork*>::const_iterator b = networks.begin(); b != networks.end(); ++b) {
+			(*b)->GetModules().UnloadAll();
+		}
 	}
 
 	for (size_t b = 0; b < m_vpListeners.size(); b++) {
@@ -261,7 +271,16 @@ bool CZNC::WritePemFile() {
 	CString sPemFile = GetPemLocation();
 
 	CUtils::PrintAction("Writing Pem file [" + sPemFile + "]");
+#ifndef _WIN32
+    int fd = creat(sPemFile.c_str(), 0600);
+	if (fd == -1) {
+		CUtils::PrintStatus(false, "Unable to open");
+		return false;
+	}
+    FILE *f = fdopen(fd, "w");
+#else
 	FILE *f = fopen(sPemFile.c_str(), "w");
+#endif
 
 	if (!f) {
 		CUtils::PrintStatus(false, "Unable to open");
@@ -526,7 +545,9 @@ CString CZNC::MakeConfigHeader() {
 		"// Do NOT edit this file while ZNC is running!\n"
 		"// Use webadmin or *controlpanel instead.\n"
 		"//\n"
-		"// Buf if you feel risky, you might want to read help on /znc saveconfig and /znc rehash.\n"
+		"// Altering this file by hand will forfeit all support.\n"
+		"//\n"
+		"// But if you feel risky, you might want to read help on /znc saveconfig and /znc rehash.\n"
 		"// Also check http://en.znc.in/wiki/Configuration\n";
 }
 
@@ -711,7 +732,7 @@ bool CZNC::WriteNewConfig(const CString& sConfigFile) {
 		GetModules().GetAvailableMods(ssUserMods);
 		size_t uNrOtherUserMods = FilterUncommonModules(ssUserMods);
 
-		if (ssUserMods.size()) {
+		if (!ssUserMods.empty()) {
 			vsLines.push_back("");
 			CUtils::PrintMessage("");
 			CUtils::PrintMessage("-- User Modules --");
@@ -770,7 +791,7 @@ bool CZNC::WriteNewConfig(const CString& sConfigFile) {
 			GetModules().GetAvailableMods(ssNetworkMods, CModInfo::NetworkModule);
 			size_t uNrOtherNetworkMods = FilterUncommonModules(ssNetworkMods);
 
-			if (ssNetworkMods.size()) {
+			if (!ssNetworkMods.empty()) {
 				CUtils::PrintMessage("");
 				CUtils::PrintMessage("-- Network Modules --");
 				CUtils::PrintMessage("");
@@ -964,7 +985,7 @@ size_t CZNC::FilterUncommonModules(set<CModInfo>& ssModules) {
 	size_t uNrRemoved = 0;
 	for(set<CModInfo>::iterator it = ssModules.begin(); it != ssModules.end(); ) {
 		if(ssNames.count(it->GetName()) > 0) {
-			it++;
+			++it;
 		} else {
 			set<CModInfo>::iterator it2 = it++;
 			ssModules.erase(it2);
